/*
 * This file is part of Cute File Manager.
 *
 * Cute File Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as published by
 * the Free Software Foundation.
 *
 * Cute File Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cute File Manager.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2008 Catalin Iacob
 */

#include "files_lister.h"
#include "filter.h"

#include <QDesktopServices>
#include <QDir>
#include <QFileInfo>
#include <QFileSystemModel>
#include <QHeaderView>
#include <QKeyEvent>
#include <QSortFilterProxyModel>
#include <QUrl>

FilesLister::FilesLister(const QString& initial_path, QWidget* parent)
	: QTableView(parent),
	  path(initial_path)
{
	dir_model = new QFileSystemModel(parent);
	dir_model->setFilter(QDir::AllEntries);
	dir_model->setRootPath(path);

	filtered_dir_model = new NoCurrentDirProxyModel(parent);
	filtered_dir_model->setDynamicSortFilter(true);
	filtered_dir_model->setSourceModel(dir_model);

	setModel(filtered_dir_model);
	setRootIndex(filtered_dir_model->mapFromSource(dir_model->index(path)));

	setShowGrid(false);
	setAlternatingRowColors(true);

	setSelectionMode(QAbstractItemView::ExtendedSelection);
	setSelectionBehavior(QAbstractItemView::SelectRows);
	//we want Tab to cycle through the panels not through each panel's items
	setTabKeyNavigation(false);

	QHeaderView* horizontal_header = horizontalHeader();
	horizontal_header->setResizeMode(QHeaderView::Stretch);

	QHeaderView* vertical_header = verticalHeader();
	vertical_header->setVisible(false);

	horizontal_header->setHighlightSections(false);
	vertical_header->setHighlightSections(false);

	connect(this, SIGNAL(activated(const QModelIndex&)), this, SLOT(open_selected(const QModelIndex&)));
}

QString FilesLister::get_path() const{
	Q_ASSERT(path == QDir::cleanPath(path));
	return path;
}

void FilesLister::open_selected(const QModelIndex& index_in_proxy){
	QModelIndex index_in_dir = filtered_dir_model->mapToSource(index_in_proxy);
	QModelIndex new_dir_index = index_in_dir.sibling(index_in_dir.row(), 0);
	Q_ASSERT(new_dir_index.isValid());

	QFileInfo selected_file_info = dir_model->fileInfo(new_dir_index);
	if(selected_file_info.isDir()){
		path = QDir::cleanPath(dir_model->filePath(new_dir_index));
		//calling setRootPath resets dir_model so we need to obtain new_dir_index again
		new_dir_index = dir_model->setRootPath(path);

		setRootIndex(filtered_dir_model->mapFromSource(new_dir_index));
		selectRow(0);
	} else if(selected_file_info.isFile()){
		QString path_to_open = selected_file_info.absoluteFilePath();
		bool success = QDesktopServices::openUrl(QUrl::fromLocalFile(path_to_open));
		if(!success)
			qCritical("(TODO replace me with proper logging!)\nUnable to open %s", qPrintable(path_to_open));
	}
}

void FilesLister::keyPressEvent(QKeyEvent* event){
	QModelIndexList selected_indexes = selectedIndexes();
	int key_pressed = event->key();
	bool interesting_key_pressed = (key_pressed == Qt::Key_Right) || (key_pressed == Qt::Key_Left);
	if(!interesting_key_pressed || selected_indexes.empty()){
		QTableView::keyPressEvent(event);
		return;
	}

	if(key_pressed == Qt::Key_Right){
		const QModelIndex& current_selection = selected_indexes.at(0);
		emit activated(current_selection);
	} else if(key_pressed == Qt::Key_Left){
		emit activated(rootIndex().parent());
	} else {
		Q_ASSERT(false);
	}
}

