/*
 * This file is part of Cute File Manager.
 *
 * Cute File Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as published by
 * the Free Software Foundation.
 *
 * Cute File Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cute File Manager.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2008 Catalin Iacob
 */

#include "files_lister.h"
#include "filesystem_operation.h"
#include "main_window.h"
#include "settings.h"

#include <QApplication>
#include <QMenuBar>
#include <QPushButton>
#include <QSignalMapper>
#include <QSplitter>
#include <QToolBar>

MainWindow::MainWindow(){
	QSplitter* two_pane_splitter = new QSplitter;

	left_lister = new FilesLister("/home/catalin", two_pane_splitter);
	right_lister = new FilesLister("c:\\Qt\\4.3.1", two_pane_splitter);

	two_pane_splitter->addWidget(left_lister);
	two_pane_splitter->addWidget(right_lister);

	create_actions();
	create_menus();
	create_fn_buttons();
	connect_fn_buttons();
	statusBar();

	setCentralWidget(two_pane_splitter);
}

void MainWindow::dispatch_file_operation(int op_idx) const{
	Q_ASSERT(FN_FILE_OPERATIONS_MIN <= op_idx);
	Q_ASSERT(op_idx < FN_FILE_OPERATIONS_MAX);

	FilesLister* current_lister = get_focused_lister();
	QModelIndexList selection_list = current_lister->selectionModel()->selectedRows();
	QStringList all_selected_names;
	foreach (QModelIndex selected_index, selection_list){
		QVariant data = selected_index.data();
		Q_ASSERT(data.canConvert(QVariant::String));
		QString current_file_name = data.toString();

		all_selected_names.append(current_file_name);
	}

	QString current_panel_path = current_lister->get_path();
	QString other_panel_path = get_other_lister(current_lister)->get_path();
	FileSystemOperation* fs_op = FileSystemOperation::create_instance(op_idx, current_panel_path, all_selected_names, other_panel_path);
	if(fs_op->show_operation_GUI_modal() == FileSystemOperation::accepted){
		fs_op->perform_file_operation();
	}
	delete fs_op;
}

void MainWindow::create_actions(){
	about_qt_action = new QAction(tr("About &Qt"), this);
	about_qt_action->setStatusTip(tr("Show Qt version"));
	connect(about_qt_action, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void MainWindow::create_menus(){
	menuBar()->addSeparator();

	help_menu = menuBar()->addMenu(tr("&Help"));
	help_menu->addAction(about_qt_action);
}

void MainWindow::create_fn_buttons(){
	QToolBar* fn_tool_bar = new QToolBar(tr("Function buttons tool bar"), this);

	for(int i = 0; i < NR_FN_BUTTONS; ++i){
		fn_buttons[i] = new QPushButton(tr("F%1 %2").arg(i + 1).arg(tr(fn_buttons_labels[i])), fn_tool_bar);
		fn_buttons[i]->setShortcut(tr("F%1").arg(i + 1));
		fn_buttons[i]->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
		//we want Tab to cycle through the panels not through the Fn buttons so we never focus them
		fn_buttons[i]->setFocusPolicy(Qt::NoFocus);

		fn_tool_bar->addWidget(fn_buttons[i]);
	}

	addToolBar(Qt::BottomToolBarArea, fn_tool_bar);
}

void MainWindow::connect_fn_buttons(){
	QSignalMapper* mapper = new QSignalMapper(this);
	for(int i = FN_FILE_OPERATIONS_MIN; i < FN_FILE_OPERATIONS_MAX; ++i){
		mapper->setMapping(fn_buttons[i], i);
		connect(fn_buttons[i], SIGNAL(clicked()), mapper, SLOT(map()));
	}
	connect(mapper, SIGNAL(mapped(int)), this, SLOT(dispatch_file_operation(int)));

	connect(fn_buttons[FN_QUIT_BUTTON_IDX], SIGNAL(clicked()), qApp, SLOT(quit()));
}

FilesLister* MainWindow::get_focused_lister() const{
	Q_ASSERT(left_lister->hasFocus() || right_lister->hasFocus());

	return left_lister->hasFocus() ? left_lister : right_lister;
}

FilesLister* MainWindow::get_other_lister(FilesLister* lister) const{
	Q_ASSERT(lister == left_lister || lister == right_lister);

	return lister == left_lister ? right_lister : left_lister;
}

